-module(megaco_text_parser).
-define(THIS_MODULE, megaco_text_parser).
-export([parse/1, parse_and_scan/1, format_error/1]).

-include("megaco_text_parser.hrl").


%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The parser generator will insert appropriate declarations before this line.%

parse(Tokens) ->
    case catch yeccpars1(Tokens, false, 0, [], []) of
	error ->
	    Errorline =
		if Tokens == [] -> 0; true -> element(2, hd(Tokens)) end,
	    {error,
	     {Errorline, ?THIS_MODULE, "syntax error at or after this line."}};
	Other ->
	    Other
    end.

parse_and_scan({Mod, Fun, Args}) ->
    case apply(Mod, Fun, Args) of
	{eof, _} ->
	    {ok, eof};
	{error, Descriptor, _} ->
	    {error, Descriptor};
	{ok, Tokens, _} ->
	    yeccpars1(Tokens, {Mod, Fun, Args}, 0, [], [])
    end.

format_error(Message) ->
    case io_lib:deep_char_list(Message) of
	true ->
	    Message;
	_ ->
	    io_lib:write(Message)
    end.

% To be used in grammar files to throw an error message to the parser toplevel.
% Doesn't have to be exported!
return_error(Line, Message) ->
    throw({error, {Line, ?THIS_MODULE, Message}}).


% Don't change yeccpars1/6 too much, it is called recursively by yeccpars2/8!
yeccpars1([Token | Tokens], Tokenizer, State, States, Vstack) ->
    yeccpars2(State, element(1, Token), States, Vstack, Token, Tokens,
	      Tokenizer);
yeccpars1([], {M, F, A}, State, States, Vstack) ->
    case catch apply(M, F, A) of
        {eof, Endline} ->
            {error, {Endline, ?THIS_MODULE, "end_of_file"}};
        {error, Descriptor, _Endline} ->
            {error, Descriptor};
        {'EXIT', Reason} ->
            {error, {0, ?THIS_MODULE, Reason}};
        {ok, Tokens, _Endline} ->
	    case catch yeccpars1(Tokens, {M, F, A}, State, States, Vstack) of
		error ->
		    Errorline = element(2, hd(Tokens)),
		    {error, {Errorline, ?THIS_MODULE,
			     "syntax error at or after this line."}};
		Other ->
		    Other
	    end
    end;
yeccpars1([], false, State, States, Vstack) ->
    yeccpars2(State, '$end', States, Vstack, {'$end', 999999}, [], false).

% For internal use only.
yeccerror(Token) ->
    {error,
     {element(2, Token), ?THIS_MODULE,
      ["syntax error before: ", yecctoken2string(Token)]}}.

yecctoken2string({atom, _, A}) -> io_lib:write(A);
yecctoken2string({integer,_,N}) -> io_lib:write(N);
yecctoken2string({float,_,F}) -> io_lib:write(F);
yecctoken2string({char,_,C}) -> io_lib:write_char(C);
yecctoken2string({var,_,V}) -> io_lib:format('~s', [V]);
yecctoken2string({string,_,S}) -> io_lib:write_string(S);
yecctoken2string({reserved_symbol, _, A}) -> io_lib:format('~w', [A]);
yecctoken2string({_Cat, _, Val}) -> io_lib:format('~w', [Val]);

yecctoken2string({'dot', _}) -> io_lib:format('~w', ['.']);
yecctoken2string({'$end', _}) ->
    [];
yecctoken2string({Other, _}) when atom(Other) ->
    io_lib:format('~w', [Other]);
yecctoken2string(Other) ->
    io_lib:write(Other).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


yeccpars2(0, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [0 | __Ss], [__T | __Stack]);
yeccpars2(0, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(3, __Cat, [0 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(1, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sep,
 yeccpars2(yeccgoto(optSep, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(2, '$end', _, __Stack, _, _, _) ->
 {ok, hd(__Stack)};
yeccpars2(2, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(3, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 4, [3 | __Ss], [__T | __Stack]);
yeccpars2(3, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(5, __Cat, [3 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(4, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 775, [4 | __Ss], [__T | __Stack]);
yeccpars2(4, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(5, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(6, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(7, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(8, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(9, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(10, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(11, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(12, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(13, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(14, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(15, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(16, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(17, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(18, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(19, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(20, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(21, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(22, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(23, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(24, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(25, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(26, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(27, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(28, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(29, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(30, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(31, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(32, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(33, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(34, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(35, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(36, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(37, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(38, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(39, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(40, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(41, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(42, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(43, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(44, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(45, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(46, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(47, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(48, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(49, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(50, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(51, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(52, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(53, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(54, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(55, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(56, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(57, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(58, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(59, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(60, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(61, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(62, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(63, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(64, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(65, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(66, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(67, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(68, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(69, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(70, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(71, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(72, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(73, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(74, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(75, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(76, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(77, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(78, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(79, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(80, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(81, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(82, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(83, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(84, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(85, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(86, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(87, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(88, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(89, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(90, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(91, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(92, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(93, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(94, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(95, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(96, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(97, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(98, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(99, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(100, 'endOfMessage', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 774, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(101, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 103, [101 | __Ss], [__T | __Stack]);
yeccpars2(101, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 102, [101 | __Ss], [__T | __Stack]);
yeccpars2(101, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [101 | __Ss], [__T | __Stack]);
yeccpars2(101, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(107, __Cat, [101 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(102, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(103, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(104, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(mId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(105, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(mId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(106, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [106 | __Ss], [__T | __Stack]);
yeccpars2(106, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 117, [106 | __Ss], [__T | __Stack]);
yeccpars2(106, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 119, [106 | __Ss], [__T | __Stack]);
yeccpars2(106, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 116, [106 | __Ss], [__T | __Stack]);
yeccpars2(106, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 118, [106 | __Ss], [__T | __Stack]);
yeccpars2(106, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(107, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, 'MtpAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [107 | __Ss], [__T | __Stack]);
yeccpars2(107, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(108, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_mtpAddress(__1),
 yeccpars2(yeccgoto(mtpAddress, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(109, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [109 | __Ss], [__T | __Stack]);
yeccpars2(109, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(114, __Cat, [109 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(110, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [110 | __Ss], [__T | __Stack]);
yeccpars2(110, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(113, __Cat, [110 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(111, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {deviceName,__1},
 yeccpars2(yeccgoto(deviceName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(112, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_pathName(__1),
 yeccpars2(yeccgoto(pathName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(113, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(114, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(115, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 748, [115 | __Ss], [__T | __Stack]);
yeccpars2(115, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(116, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 760, [116 | __Ss], [__T | __Stack]);
yeccpars2(116, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(117, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 618, [117 | __Ss], [__T | __Stack]);
yeccpars2(117, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(118, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 610, [118 | __Ss], [__T | __Stack]);
yeccpars2(118, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(119, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 129, [119 | __Ss], [__T | __Stack]);
yeccpars2(119, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 130, [119 | __Ss], [__T | __Stack]);
yeccpars2(119, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(120, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {messageError,__1},
 yeccpars2(yeccgoto(messageBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(121, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_message(__1,__2,__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(message, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(122, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 117, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 119, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 116, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 118, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1],
 yeccpars2(yeccgoto(transactionList, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(123, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactions,__1},
 yeccpars2(yeccgoto(messageBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(124, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionPending,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(125, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionReply,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(126, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionRequest,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(127, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionResponseAck,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(128, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(transactionList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(129, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 600, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(130, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 131, [130 | __Ss], [__T | __Stack]);
yeccpars2(130, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(131, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 138, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(132, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 133, [132 | __Ss], [__T | __Stack]);
yeccpars2(132, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(134, __Cat, [132 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(133, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 131, [133 | __Ss], [__T | __Stack]);
yeccpars2(133, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(134, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 135, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(135, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = asn1_NOVALUE, actions = [__3|__4]},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(136, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 133, [136 | __Ss], [__T | __Stack]);
yeccpars2(136, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(137, __Cat, [136 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(137, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionRequestList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(138, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(139, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 141, [139 | __Ss], [__T | __Stack]);
yeccpars2(139, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(140, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_contextID(__1),
 yeccpars2(yeccgoto(contextID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(141, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 150, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 153, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 151, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 149, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 143, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 152, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 148, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 142, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 145, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [141 | __Ss], [__T | __Stack]);
yeccpars2(141, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(142, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {addReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(143, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 597, [143 | __Ss], [__T | __Stack]);
yeccpars2(143, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(144, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 594, [144 | __Ss], [__T | __Stack]);
yeccpars2(144, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(145, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 584, [145 | __Ss], [__T | __Stack]);
yeccpars2(145, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(146, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {emergency,true},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(147, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {modReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(148, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {moveReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(149, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 567, [149 | __Ss], [__T | __Stack]);
yeccpars2(149, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(150, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 565, [150 | __Ss], [__T | __Stack]);
yeccpars2(150, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(151, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 517, [151 | __Ss], [__T | __Stack]);
yeccpars2(151, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(152, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 511, [152 | __Ss], [__T | __Stack]);
yeccpars2(152, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(153, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 494, [153 | __Ss], [__T | __Stack]);
yeccpars2(153, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(154, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 489, [154 | __Ss], [__T | __Stack]);
yeccpars2(154, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(490, __Cat, [154 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(155, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(156, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [156 | __Ss], [__T | __Stack]);
yeccpars2(156, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(157, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(158, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(159, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(160, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(161, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(162, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {priority,__1},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(163, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(164, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(165, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {topology,__1},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(166, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [166 | __Ss], [__T | __Stack]);
yeccpars2(166, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(167, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_terminationID(__1),
 yeccpars2(yeccgoto(terminationID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(168, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 169, [168 | __Ss], [__T | __Stack]);
yeccpars2(168, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(170, __Cat, [168 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(169, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 176, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 178, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 174, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 177, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 173, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 175, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 171, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(170, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest(__1,#'AmmRequest'{terminationID = [__3], descriptors = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(171, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 471, [171 | __Ss], [__T | __Stack]);
yeccpars2(171, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(172, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  value_of(__1),
 yeccpars2(yeccgoto(digitMapDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(173, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 445, [173 | __Ss], [__T | __Stack]);
yeccpars2(173, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(174, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 381, [174 | __Ss], [__T | __Stack]);
yeccpars2(174, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(175, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 311, [175 | __Ss], [__T | __Stack]);
yeccpars2(175, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(176, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 291, [176 | __Ss], [__T | __Stack]);
yeccpars2(176, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 290, [176 | __Ss], [__T | __Stack]);
yeccpars2(176, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(177, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 279, [177 | __Ss], [__T | __Stack]);
yeccpars2(177, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(178, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 193, [178 | __Ss], [__T | __Stack]);
yeccpars2(178, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(179, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 188, [179 | __Ss], [__T | __Stack]);
yeccpars2(179, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(189, __Cat, [179 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(180, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(181, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {digitMapDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(182, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(183, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventsDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(184, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mediaDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(185, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {modemDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(186, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {muxDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(187, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signalsDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(188, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 176, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 178, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 174, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 177, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 173, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 175, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 171, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(189, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 190, [189 | __Ss], [__T | __Stack]);
yeccpars2(189, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(190, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammRequestBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(191, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 188, [191 | __Ss], [__T | __Stack]);
yeccpars2(191, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(192, __Cat, [191 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(192, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(ammParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(193, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 195, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 194, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(194, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(signalsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(195, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 268, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(196, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(signalName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(197, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_pkgdName(__1),
 yeccpars2(yeccgoto(pkgdName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(198, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {seqSigList,__1},
 yeccpars2(yeccgoto(signalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(199, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 207, [199 | __Ss], [__T | __Stack]);
yeccpars2(199, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_signalRequest(__1,[]),
 yeccpars2(yeccgoto(signalRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(200, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 202, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(203, __Cat, [200 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(201, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signal,__1},
 yeccpars2(yeccgoto(signalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(202, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 195, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [202 | __Ss], [__T | __Stack]);
yeccpars2(202, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(203, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 204, [203 | __Ss], [__T | __Stack]);
yeccpars2(203, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(204, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(signalsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(205, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 202, [205 | __Ss], [__T | __Stack]);
yeccpars2(205, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(206, __Cat, [205 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(206, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(signalParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(207, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 212, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 211, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 210, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 209, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 208, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(208, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 266, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(209, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(sigParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(209, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(sigParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(209, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(210, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 254, [210 | __Ss], [__T | __Stack]);
yeccpars2(210, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(211, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 249, [211 | __Ss], [__T | __Stack]);
yeccpars2(211, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(212, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 246, [212 | __Ss], [__T | __Stack]);
yeccpars2(212, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(213, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 221, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 222, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 223, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 220, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(214, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(216, __Cat, [214 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(215, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 212, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 211, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 210, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 209, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 208, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(216, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 217, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(217, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_signalRequest(__1,[__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(signalRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(218, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [218 | __Ss], [__T | __Stack]);
yeccpars2(218, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(219, __Cat, [218 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(219, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(220, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 231, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 230, [220 | __Ss], [__T | __Stack]);
yeccpars2(220, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(221, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [221 | __Ss], [__T | __Stack]);
yeccpars2(221, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(222, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [222 | __Ss], [__T | __Stack]);
yeccpars2(222, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(223, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(224, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {other,ensure_NAME(__1),__2},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(225, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_value(__1),
 yeccpars2(yeccgoto(value, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(226, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_value(__1),
 yeccpars2(yeccgoto(value, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(227, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,unequalTo}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(228, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,smallerThan}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(229, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,greaterThan}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(230, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(231, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [231 | __Ss], [__T | __Stack]);
yeccpars2(231, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(232, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(233, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__1]},
 yeccpars2(yeccgoto(alternativeValue, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(234, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 236, [234 | __Ss], [__T | __Stack]);
yeccpars2(234, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 235, [234 | __Ss], [__T | __Stack]);
yeccpars2(234, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(237, __Cat, [234 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(235, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [235 | __Ss], [__T | __Stack]);
yeccpars2(235, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(236, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(237, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 238, [237 | __Ss], [__T | __Stack]);
yeccpars2(237, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(238, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2|__3], extraInfo = {sublist,true}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(239, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 236, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(240, __Cat, [239 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(240, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(valueList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(241, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 242, [241 | __Ss], [__T | __Stack]);
yeccpars2(241, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(242, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2,__4], extraInfo = {range,true}},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(243, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 236, [243 | __Ss], [__T | __Stack]);
yeccpars2(243, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(244, __Cat, [243 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(244, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 245, [244 | __Ss], [__T | __Stack]);
yeccpars2(244, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(245, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2|__3], extraInfo = {sublist,false}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(246, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [246 | __Ss], [__T | __Stack]);
yeccpars2(246, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(247, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_streamID(__1),
 yeccpars2(yeccgoto(streamID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(248, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {stream,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(249, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 250, [249 | __Ss], [__T | __Stack]);
yeccpars2(249, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 252, [249 | __Ss], [__T | __Stack]);
yeccpars2(249, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 251, [249 | __Ss], [__T | __Stack]);
yeccpars2(249, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(250, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  brief,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(251, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onOff,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(252, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  timeOut,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(253, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signal_type,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(254, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 255, [254 | __Ss], [__T | __Stack]);
yeccpars2(254, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(255, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 258, [255 | __Ss], [__T | __Stack]);
yeccpars2(255, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 257, [255 | __Ss], [__T | __Stack]);
yeccpars2(255, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 256, [255 | __Ss], [__T | __Stack]);
yeccpars2(255, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 259, [255 | __Ss], [__T | __Stack]);
yeccpars2(255, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(256, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onInterruptByEvent,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(257, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onInterruptByNewSignalDescr,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(258, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  otherReason,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(259, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onTimeOut,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(260, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 261, [260 | __Ss], [__T | __Stack]);
yeccpars2(260, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(262, __Cat, [260 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(261, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 258, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 257, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 256, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 259, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(262, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 263, [262 | __Ss], [__T | __Stack]);
yeccpars2(262, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(263, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {notify_completion,[__4|__5]},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(264, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 261, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(265, __Cat, [264 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(265, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(notificationReasons, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(266, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [266 | __Ss], [__T | __Stack]);
yeccpars2(266, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(267, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {duration,ensure_uint16(__3)},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(268, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(269, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__1),
 yeccpars2(yeccgoto(signalListId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(270, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 271, [270 | __Ss], [__T | __Stack]);
yeccpars2(270, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(271, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [271 | __Ss], [__T | __Stack]);
yeccpars2(271, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(272, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 274, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(275, __Cat, [272 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(273, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(signalListParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(274, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [274 | __Ss], [__T | __Stack]);
yeccpars2(274, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(275, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 276, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(276, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'SeqSigList'{id = ensure_uint16(__3), signalList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(signalList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(277, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 274, [277 | __Ss], [__T | __Stack]);
yeccpars2(277, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(278, __Cat, [277 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(278, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(signalListParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(279, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(280, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 282, [280 | __Ss], [__T | __Stack]);
yeccpars2(280, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(281, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_muxType(__1),
 yeccpars2(yeccgoto(muxType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(282, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [282 | __Ss], [__T | __Stack]);
yeccpars2(282, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(283, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'MuxDescriptor'{muxType = __3, termList = __4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(muxDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(284, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 285, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(286, __Cat, [284 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(285, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(286, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 287, [286 | __Ss], [__T | __Stack]);
yeccpars2(286, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(287, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(terminationIDList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(288, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 285, [288 | __Ss], [__T | __Stack]);
yeccpars2(288, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(289, __Cat, [288 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(289, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(terminationIDListRepeat, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(290, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(291, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [291 | __Ss], [__T | __Stack]);
yeccpars2(291, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(292, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 294, [292 | __Ss], [__T | __Stack]);
yeccpars2(292, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(295, __Cat, [292 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(293, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_modemType(__1),
 yeccpars2(yeccgoto(modemType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(294, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(295, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 296, [295 | __Ss], [__T | __Stack]);
yeccpars2(295, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(296, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 297, [296 | __Ss], [__T | __Stack]);
yeccpars2(296, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(298, __Cat, [296 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(297, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(298, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ModemDescriptor'{mtl = [__3|__4], mpl = __6},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(modemDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(299, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 221, [299 | __Ss], [__T | __Stack]);
yeccpars2(299, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 222, [299 | __Ss], [__T | __Stack]);
yeccpars2(299, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 223, [299 | __Ss], [__T | __Stack]);
yeccpars2(299, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 220, [299 | __Ss], [__T | __Stack]);
yeccpars2(299, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(300, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 301, [300 | __Ss], [__T | __Stack]);
yeccpars2(300, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(302, __Cat, [300 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(301, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [301 | __Ss], [__T | __Stack]);
yeccpars2(301, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(302, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 303, [302 | __Ss], [__T | __Stack]);
yeccpars2(302, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(303, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(optParmValues, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(304, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 301, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(305, __Cat, [304 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(305, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(parmValues, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(306, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'PropertyParm'.name,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(propertyParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(307, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 294, [307 | __Ss], [__T | __Stack]);
yeccpars2(307, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(308, __Cat, [307 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(308, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(modemTypeList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(309, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 297, [309 | __Ss], [__T | __Stack]);
yeccpars2(309, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(310, __Cat, [309 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(310, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ModemDescriptor'{mtl = [__3], mpl = __4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(modemDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(311, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 316, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 312, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 315, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 314, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 313, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(312, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 357, [312 | __Ss], [__T | __Stack]);
yeccpars2(312, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(313, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {local,#'LocalRemoteDescriptor'{propGrps = ensure_prop_groups(__1)}},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(314, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {remote,#'LocalRemoteDescriptor'{propGrps = ensure_prop_groups(__1)}},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(315, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 348, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(316, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 327, [316 | __Ss], [__T | __Stack]);
yeccpars2(316, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(317, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {control,__1},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(318, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 322, [318 | __Ss], [__T | __Stack]);
yeccpars2(318, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(323, __Cat, [318 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(319, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamDescriptor,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(320, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamParm,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(321, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {termState,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(322, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 316, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 312, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 315, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 314, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 313, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(323, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 324, [323 | __Ss], [__T | __Stack]);
yeccpars2(323, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(324, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_mediaDescriptor([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(mediaDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(325, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 322, [325 | __Ss], [__T | __Stack]);
yeccpars2(325, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(326, __Cat, [325 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(326, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mediaParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(327, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 329, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 328, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(328, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 344, [328 | __Ss], [__T | __Stack]);
yeccpars2(328, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(329, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 339, [329 | __Ss], [__T | __Stack]);
yeccpars2(329, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(330, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferControl,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(331, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {propertyParm,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(332, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceState,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(333, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 334, [333 | __Ss], [__T | __Stack]);
yeccpars2(333, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(335, __Cat, [333 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(334, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 329, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 328, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [334 | __Ss], [__T | __Stack]);
yeccpars2(334, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(335, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 336, [335 | __Ss], [__T | __Stack]);
yeccpars2(335, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(336, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_terminationStateDescriptor([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(terminationStateDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(337, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 334, [337 | __Ss], [__T | __Stack]);
yeccpars2(337, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(338, __Cat, [337 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(338, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(terminationStateParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(339, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 340, [339 | __Ss], [__T | __Stack]);
yeccpars2(339, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 341, [339 | __Ss], [__T | __Stack]);
yeccpars2(339, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 342, [339 | __Ss], [__T | __Stack]);
yeccpars2(339, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(340, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  inSvc,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(341, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  outOfSvc,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(342, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  test,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(343, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceStates, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(344, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 345, [344 | __Ss], [__T | __Stack]);
yeccpars2(344, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 346, [344 | __Ss], [__T | __Stack]);
yeccpars2(344, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(345, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  lockStep,
 yeccpars2(yeccgoto(eventBufferControlState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(346, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  off,
 yeccpars2(yeccgoto(eventBufferControlState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(347, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventBufferControl, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(348, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(349, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 350, [349 | __Ss], [__T | __Stack]);
yeccpars2(349, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(350, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 312, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 314, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 313, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(351, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 352, [351 | __Ss], [__T | __Stack]);
yeccpars2(351, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(353, __Cat, [351 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(352, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 312, [352 | __Ss], [__T | __Stack]);
yeccpars2(352, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 314, [352 | __Ss], [__T | __Stack]);
yeccpars2(352, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 313, [352 | __Ss], [__T | __Stack]);
yeccpars2(352, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(353, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 354, [353 | __Ss], [__T | __Stack]);
yeccpars2(353, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(354, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StreamDescriptor'{streamID = __3, streamParms = merge_streamParms([__5|__6])},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(streamDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(355, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 352, [355 | __Ss], [__T | __Stack]);
yeccpars2(355, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(356, __Cat, [355 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(356, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(streamParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(357, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 360, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 359, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 358, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(358, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 374, [358 | __Ss], [__T | __Stack]);
yeccpars2(358, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(359, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 372, [359 | __Ss], [__T | __Stack]);
yeccpars2(359, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(360, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 368, [360 | __Ss], [__T | __Stack]);
yeccpars2(360, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(361, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 363, [361 | __Ss], [__T | __Stack]);
yeccpars2(361, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(364, __Cat, [361 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(362, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {prop,__1},
 yeccpars2(yeccgoto(localParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(363, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 360, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 359, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 358, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [363 | __Ss], [__T | __Stack]);
yeccpars2(363, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(364, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 365, [364 | __Ss], [__T | __Stack]);
yeccpars2(364, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(365, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(localControlDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(366, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 363, [366 | __Ss], [__T | __Stack]);
yeccpars2(366, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(367, __Cat, [366 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(367, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(368, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 369, [368 | __Ss], [__T | __Stack]);
yeccpars2(368, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 370, [368 | __Ss], [__T | __Stack]);
yeccpars2(368, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(369, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  false,
 yeccpars2(yeccgoto(onOrOff, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(370, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  true,
 yeccpars2(yeccgoto(onOrOff, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(371, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {value,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(372, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 369, [372 | __Ss], [__T | __Stack]);
yeccpars2(372, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 370, [372 | __Ss], [__T | __Stack]);
yeccpars2(372, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(373, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {group,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(374, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 376, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 375, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 379, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 377, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 378, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(375, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  inactive,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(376, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  loopBack,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(377, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  recvOnly,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(378, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sendOnly,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(379, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sendRecv,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(380, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mode,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(381, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(382, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 384, [382 | __Ss], [__T | __Stack]);
yeccpars2(382, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(383, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_requestID(__1),
 yeccpars2(yeccgoto(requestID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(384, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [384 | __Ss], [__T | __Stack]);
yeccpars2(384, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(385, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 392, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'RequestedEvent'{evParList = []},
 yeccpars2(393, __Cat, [385 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(386, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [386 | __Ss], [__T | __Stack]);
yeccpars2(386, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(388, __Cat, [386 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(387, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [387 | __Ss], [__T | __Stack]);
yeccpars2(387, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(388, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 389, [388 | __Ss], [__T | __Stack]);
yeccpars2(388, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(389, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'EventsDescriptor'{requestID = __3, eventList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(eventsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(390, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [390 | __Ss], [__T | __Stack]);
yeccpars2(390, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(391, __Cat, [390 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(391, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(requestedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(392, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 396, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 395, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 394, [392 | __Ss], [__T | __Stack]);
yeccpars2(392, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(393, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'RequestedEvent'.pkgdName,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(requestedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(394, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_eventDM(__1),
 yeccpars2(yeccgoto(eventDM, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(395, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 410, [395 | __Ss], [__T | __Stack]);
yeccpars2(395, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(396, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(396, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(396, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(397, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(398, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(399, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(400, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 405, [400 | __Ss], [__T | __Stack]);
yeccpars2(400, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(406, __Cat, [400 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(401, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 221, [401 | __Ss], [__T | __Stack]);
yeccpars2(401, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 222, [401 | __Ss], [__T | __Stack]);
yeccpars2(401, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 223, [401 | __Ss], [__T | __Stack]);
yeccpars2(401, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 220, [401 | __Ss], [__T | __Stack]);
yeccpars2(401, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(402, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(403, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_NAME(__1),
 yeccpars2(yeccgoto(eventParameterName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(404, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  select_stream_or_other(__1,__2),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(eventStreamOrOther, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(405, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 396, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 395, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 394, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(406, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 407, [406 | __Ss], [__T | __Stack]);
yeccpars2(406, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(407, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_eventParameters([__2|__3]),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(requestedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(408, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 405, [408 | __Ss], [__T | __Stack]);
yeccpars2(408, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(409, __Cat, [408 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(409, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(410, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 178, [410 | __Ss], [__T | __Stack]);
yeccpars2(410, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 411, [410 | __Ss], [__T | __Stack]);
yeccpars2(410, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(411, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 419, [411 | __Ss], [__T | __Stack]);
yeccpars2(411, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(412, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 418, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(413, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 415, [413 | __Ss], [__T | __Stack]);
yeccpars2(413, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 414, [413 | __Ss], [__T | __Stack]);
yeccpars2(413, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(414, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 411, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(415, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,__3,asn1_NOVALUE},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedWithSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(416, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 417, [416 | __Ss], [__T | __Stack]);
yeccpars2(416, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(417, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,__3,__5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(embedWithSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(418, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,asn1_NOVALUE,__3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedNoSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(419, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [419 | __Ss], [__T | __Stack]);
yeccpars2(419, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(420, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 421, [420 | __Ss], [__T | __Stack]);
yeccpars2(420, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(421, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [421 | __Ss], [__T | __Stack]);
yeccpars2(421, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(422, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 429, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'SecondRequestedEvent'{evParList = []},
 yeccpars2(430, __Cat, [422 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(423, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 424, [423 | __Ss], [__T | __Stack]);
yeccpars2(423, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(425, __Cat, [423 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(424, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(425, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 426, [425 | __Ss], [__T | __Stack]);
yeccpars2(425, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(426, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'SecondEventsDescriptor'{requestID = __3, eventList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(embedFirst, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(427, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 424, [427 | __Ss], [__T | __Stack]);
yeccpars2(427, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(428, __Cat, [427 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(428, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(secondRequestedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(429, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 432, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 431, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 394, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(430, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'SecondRequestedEvent'.pkgdName,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(secondRequestedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(431, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 442, [431 | __Ss], [__T | __Stack]);
yeccpars2(431, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(432, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(432, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(432, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(433, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(434, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(435, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(436, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 437, [436 | __Ss], [__T | __Stack]);
yeccpars2(436, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(438, __Cat, [436 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(437, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 432, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 431, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 394, [437 | __Ss], [__T | __Stack]);
yeccpars2(437, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(438, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [438 | __Ss], [__T | __Stack]);
yeccpars2(438, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(439, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_secondEventParameters([__2|__3]),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(secondRequestedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(440, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 437, [440 | __Ss], [__T | __Stack]);
yeccpars2(440, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(441, __Cat, [440 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(441, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(secondEventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(442, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 178, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(443, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 444, [443 | __Ss], [__T | __Stack]);
yeccpars2(443, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(444, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {second_embed,__3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(445, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(448, __Cat, [445 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(446, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(467, __Cat, [446 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(447, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_eventSpec(__1),
 yeccpars2(yeccgoto(eventSpec, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(448, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(463, __Cat, [448 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(449, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 454, [449 | __Ss], [__T | __Stack]);
yeccpars2(449, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(455, __Cat, [449 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(450, 'COLON', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'COLON', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(450, 'SEP', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'SEP', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(450, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_pkgdName(__1),
 yeccpars2(yeccgoto(pkgdName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(451, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [451 | __Ss], [__T | __Stack]);
yeccpars2(451, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(452, __Cat, [451 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(452, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 453, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(453, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEventTimeStamp, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(454, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [454 | __Ss], [__T | __Stack]);
yeccpars2(454, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(455, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_observed_event(__2,__1,asn1_NOVALUE),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(observedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(456, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(observedEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(457, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 458, [457 | __Ss], [__T | __Stack]);
yeccpars2(457, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(459, __Cat, [457 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(458, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [458 | __Ss], [__T | __Stack]);
yeccpars2(458, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(459, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 460, [459 | __Ss], [__T | __Stack]);
yeccpars2(459, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(460, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(observedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(461, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 458, [461 | __Ss], [__T | __Stack]);
yeccpars2(461, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(462, __Cat, [461 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(462, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(463, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [463 | __Ss], [__T | __Stack]);
yeccpars2(463, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(464, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 454, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(465, __Cat, [464 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(465, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_observed_event(__4,__3,__1),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(observedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(466, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [466 | __Ss], [__T | __Stack]);
yeccpars2(466, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(448, __Cat, [466 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(467, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 468, [467 | __Ss], [__T | __Stack]);
yeccpars2(467, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(468, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(eventBufferDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(469, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [469 | __Ss], [__T | __Stack]);
yeccpars2(469, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(470, __Cat, [469 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(470, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventSpecList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(471, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 479, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 478, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 481, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 480, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 476, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 477, [471 | __Ss], [__T | __Stack]);
yeccpars2(471, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(482, __Cat, [471 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(472, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  digitMapToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(473, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventBufferToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(474, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(475, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  mediaToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(476, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modemToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(477, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  muxToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(478, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  observedEventsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(479, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  packagesToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(480, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  signalsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(481, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  statsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(482, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 488, [482 | __Ss], [__T | __Stack]);
yeccpars2(482, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(483, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 484, [483 | __Ss], [__T | __Stack]);
yeccpars2(483, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(485, __Cat, [483 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(484, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 479, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 478, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 481, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 480, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 476, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 477, [484 | __Ss], [__T | __Stack]);
yeccpars2(484, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(485, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(auditDescriptorBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(486, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 484, [486 | __Ss], [__T | __Stack]);
yeccpars2(486, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(487, __Cat, [486 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(487, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditItemList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(488, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'AuditDescriptor'{auditToken = __3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(489, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 150, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 153, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 151, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 149, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 143, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 152, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 148, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 142, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 145, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [489 | __Ss], [__T | __Stack]);
yeccpars2(489, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(490, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 491, [490 | __Ss], [__T | __Stack]);
yeccpars2(490, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(491, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_action_requests(__3,[__5|__6]),
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(actionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(492, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 489, [492 | __Ss], [__T | __Stack]);
yeccpars2(492, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(493, __Cat, [492 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(493, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionRequestItems, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(494, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(495, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 503, [495 | __Ss], [__T | __Stack]);
yeccpars2(495, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(496, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(terminationA, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(497, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 498, [497 | __Ss], [__T | __Stack]);
yeccpars2(497, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(499, __Cat, [497 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(498, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(499, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 500, [499 | __Ss], [__T | __Stack]);
yeccpars2(499, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(500, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(topologyDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(501, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 498, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(502, __Cat, [501 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(502, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(topologyTripleList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(503, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [503 | __Ss], [__T | __Stack]);
yeccpars2(503, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(504, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 506, [504 | __Ss], [__T | __Stack]);
yeccpars2(504, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(505, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(terminationB, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(506, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 509, [506 | __Ss], [__T | __Stack]);
yeccpars2(506, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 508, [506 | __Ss], [__T | __Stack]);
yeccpars2(506, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 507, [506 | __Ss], [__T | __Stack]);
yeccpars2(506, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(507, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  bothway,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(508, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  isolate,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(509, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  oneway,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(510, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TopologyRequest'{terminationFrom = __1, terminationTo = __3, topologyDirection = __5},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(topologyTriple, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(511, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(512, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 513, [512 | __Ss], [__T | __Stack]);
yeccpars2(512, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(514, __Cat, [512 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(513, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 171, [513 | __Ss], [__T | __Stack]);
yeccpars2(513, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(514, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({subtractReq,__1},#'SubtractRequest'{terminationID = [__3], auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(subtractRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(515, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 516, [515 | __Ss], [__T | __Stack]);
yeccpars2(515, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(516, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(optAuditDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(517, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [517 | __Ss], [__T | __Stack]);
yeccpars2(517, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(518, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 519, [518 | __Ss], [__T | __Stack]);
yeccpars2(518, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(519, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 520, [519 | __Ss], [__T | __Stack]);
yeccpars2(519, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(520, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 523, [520 | __Ss], [__T | __Stack]);
yeccpars2(520, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(521, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 522, [521 | __Ss], [__T | __Stack]);
yeccpars2(521, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(522, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({serviceChangeReq,__1},#'ServiceChangeRequest'{terminationID = [__3], serviceChangeParms = __5}),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(serviceChangeRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(523, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 530, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 529, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 528, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 527, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 526, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 525, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 524, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(524, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 563, [524 | __Ss], [__T | __Stack]);
yeccpars2(524, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(525, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 561, [525 | __Ss], [__T | __Stack]);
yeccpars2(525, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(526, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 559, [526 | __Ss], [__T | __Stack]);
yeccpars2(526, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(527, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 557, [527 | __Ss], [__T | __Stack]);
yeccpars2(527, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(528, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 555, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(529, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 551, [529 | __Ss], [__T | __Stack]);
yeccpars2(529, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(530, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 549, [530 | __Ss], [__T | __Stack]);
yeccpars2(530, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(531, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {extension,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(532, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 221, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 222, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 223, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 220, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(533, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(533, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(533, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_extensionParameter(__1),
 yeccpars2(yeccgoto(extensionParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(534, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {address,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(535, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {delay,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(536, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {method,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(537, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mgc_id,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(538, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 543, [538 | __Ss], [__T | __Stack]);
yeccpars2(538, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'ServiceChangeParm'{},
 yeccpars2(544, __Cat, [538 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(539, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {profile,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(540, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {reason,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(541, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {version,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(542, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {time_stamp,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(543, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 530, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 529, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 528, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 527, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 526, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 525, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 524, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(544, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 545, [544 | __Ss], [__T | __Stack]);
yeccpars2(544, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(545, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeParm(__3,__4),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(serviceChangeDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(546, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 543, [546 | __Ss], [__T | __Stack]);
yeccpars2(546, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'ServiceChangeParm'{},
 yeccpars2(547, __Cat, [546 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(547, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeParm(__2,__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(548, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'PropertyParm'.name,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(extension, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(549, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [549 | __Ss], [__T | __Stack]);
yeccpars2(549, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(550, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_version(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeVersion, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(551, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 103, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 102, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(107, __Cat, [551 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(552, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(553, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {portNumber,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(554, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__1),
 yeccpars2(yeccgoto(portNumber, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(555, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [555 | __Ss], [__T | __Stack]);
yeccpars2(555, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(556, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReason, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(557, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [557 | __Ss], [__T | __Stack]);
yeccpars2(557, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(558, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_profile(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeProfile, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(559, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 103, [559 | __Ss], [__T | __Stack]);
yeccpars2(559, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 102, [559 | __Ss], [__T | __Stack]);
yeccpars2(559, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [559 | __Ss], [__T | __Stack]);
yeccpars2(559, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(107, __Cat, [559 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(560, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeMgcId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(561, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(562, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_serviceChangeMethod(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeMethod, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(563, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [563 | __Ss], [__T | __Stack]);
yeccpars2(563, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(564, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint32(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeDelay, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(565, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [565 | __Ss], [__T | __Stack]);
yeccpars2(565, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(566, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(priority, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(567, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [567 | __Ss], [__T | __Stack]);
yeccpars2(567, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(568, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 569, [568 | __Ss], [__T | __Stack]);
yeccpars2(568, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(569, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 570, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(570, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 575, [570 | __Ss], [__T | __Stack]);
yeccpars2(570, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(571, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'NotifyRequest'{errorDescriptor = __1},
 yeccpars2(yeccgoto(notifyRequestBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(572, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 574, [572 | __Ss], [__T | __Stack]);
yeccpars2(572, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(573, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'NotifyRequest'{observedEventsDescriptor = __1},
 yeccpars2(yeccgoto(notifyRequestBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(574, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({notifyReq,__1},setelement(#'NotifyRequest'.terminationID,__5,[__3])),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(notifyRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(575, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(576, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 577, [576 | __Ss], [__T | __Stack]);
yeccpars2(576, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(577, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(448, __Cat, [577 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(578, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 579, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(580, __Cat, [578 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(579, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(448, __Cat, [579 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(580, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 581, [580 | __Ss], [__T | __Stack]);
yeccpars2(580, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(581, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ObservedEventsDescriptor'{requestId = __3, observedEventLst = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(observedEventsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(582, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 579, [582 | __Ss], [__T | __Stack]);
yeccpars2(582, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(583, __Cat, [582 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(583, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(584, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 586, [584 | __Ss], [__T | __Stack]);
yeccpars2(584, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 585, [584 | __Ss], [__T | __Stack]);
yeccpars2(584, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 587, [584 | __Ss], [__T | __Stack]);
yeccpars2(584, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(585, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  emergencyAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(586, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  priorityAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(587, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  topologyAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(588, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 589, [588 | __Ss], [__T | __Stack]);
yeccpars2(588, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(590, __Cat, [588 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(589, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 586, [589 | __Ss], [__T | __Stack]);
yeccpars2(589, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 585, [589 | __Ss], [__T | __Stack]);
yeccpars2(589, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 587, [589 | __Ss], [__T | __Stack]);
yeccpars2(589, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(590, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 591, [590 | __Ss], [__T | __Stack]);
yeccpars2(590, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(591, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(contextAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(592, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 589, [592 | __Ss], [__T | __Stack]);
yeccpars2(592, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(593, __Cat, [592 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(593, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(contextAuditProperties, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(594, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [594 | __Ss], [__T | __Stack]);
yeccpars2(594, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(595, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 513, [595 | __Ss], [__T | __Stack]);
yeccpars2(595, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(596, __Cat, [595 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(596, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({auditValueRequest,__1},#'AuditRequest'{terminationID = __3, auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(597, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(598, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 513, [598 | __Ss], [__T | __Stack]);
yeccpars2(598, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(599, __Cat, [598 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(599, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({auditCapRequest,__1},#'AuditRequest'{terminationID = __3, auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(600, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 131, [600 | __Ss], [__T | __Stack]);
yeccpars2(600, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(601, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint32(__1),
 yeccpars2(yeccgoto(transactionID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(602, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 603, [602 | __Ss], [__T | __Stack]);
yeccpars2(602, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(603, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 131, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(604, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 133, [604 | __Ss], [__T | __Stack]);
yeccpars2(604, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(605, __Cat, [604 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(605, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 606, [605 | __Ss], [__T | __Stack]);
yeccpars2(605, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(606, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = ensure_transactionID(__3), actions = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(607, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 133, [607 | __Ss], [__T | __Stack]);
yeccpars2(607, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(608, __Cat, [607 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(608, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 609, [608 | __Ss], [__T | __Stack]);
yeccpars2(608, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(609, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = asn1_NOVALUE, actions = [__4|__5]},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(610, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [610 | __Ss], [__T | __Stack]);
yeccpars2(610, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(611, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_transactionAck(__1),
 yeccpars2(yeccgoto(transactionAck, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(612, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 613, [612 | __Ss], [__T | __Stack]);
yeccpars2(612, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(614, __Cat, [612 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(613, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [613 | __Ss], [__T | __Stack]);
yeccpars2(613, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(614, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 615, [614 | __Ss], [__T | __Stack]);
yeccpars2(614, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(615, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionResponseAck, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(616, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 613, [616 | __Ss], [__T | __Stack]);
yeccpars2(616, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(617, __Cat, [616 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(617, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(transactionAckList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(618, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [618 | __Ss], [__T | __Stack]);
yeccpars2(618, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(619, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 620, [619 | __Ss], [__T | __Stack]);
yeccpars2(619, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(620, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 621, [620 | __Ss], [__T | __Stack]);
yeccpars2(620, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(622, __Cat, [620 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(621, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 759, [621 | __Ss], [__T | __Stack]);
yeccpars2(621, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(622, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [622 | __Ss], [__T | __Stack]);
yeccpars2(622, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 623, [622 | __Ss], [__T | __Stack]);
yeccpars2(622, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(623, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 632, [623 | __Ss], [__T | __Stack]);
yeccpars2(623, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(624, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 628, [624 | __Ss], [__T | __Stack]);
yeccpars2(624, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(629, __Cat, [624 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(625, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionError,__1},
 yeccpars2(yeccgoto(transactionReplyBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(626, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 627, [626 | __Ss], [__T | __Stack]);
yeccpars2(626, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(627, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionReply'{transactionId = __3, immAckRequired = __5, transactionResult = __6},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(transactionReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(628, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 623, [628 | __Ss], [__T | __Stack]);
yeccpars2(628, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(629, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {actionReplies,[__1|__2]},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(transactionReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(630, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 628, [630 | __Ss], [__T | __Stack]);
yeccpars2(630, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(631, __Cat, [630 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(631, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionReplyList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(632, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [632 | __Ss], [__T | __Stack]);
yeccpars2(632, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(633, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 634, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(634, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 150, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 153, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 641, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 640, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 636, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 637, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 642, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 638, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 639, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 635, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [634 | __Ss], [__T | __Stack]);
yeccpars2(634, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(635, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  addReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(636, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 755, [636 | __Ss], [__T | __Stack]);
yeccpars2(636, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(637, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 735, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(638, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(639, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  moveReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(640, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 729, [640 | __Ss], [__T | __Stack]);
yeccpars2(640, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(641, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 707, [641 | __Ss], [__T | __Stack]);
yeccpars2(641, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(642, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  subtractReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(643, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 706, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(644, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(645, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 656, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(646, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(647, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 652, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(653, __Cat, [647 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(648, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {context,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(649, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ActionReply'{errorDescriptor = __1},
 yeccpars2(yeccgoto(actionReplyBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(650, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(651, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(652, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 150, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 153, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 641, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 640, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 636, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 637, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 642, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 638, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 639, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 635, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(653, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_action_reply([__1|__2]),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(actionReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(654, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 652, [654 | __Ss], [__T | __Stack]);
yeccpars2(654, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(655, __Cat, [654 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(655, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(commandReplyList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(656, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [656 | __Ss], [__T | __Stack]);
yeccpars2(656, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(657, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 658, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(659, __Cat, [657 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(658, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 668, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 666, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 663, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 665, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 667, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 661, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 664, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 660, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 662, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(659, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {__1,#'AmmsReply'{terminationID = [__3], terminationAudit = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammsReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(660, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 445, [660 | __Ss], [__T | __Stack]);
yeccpars2(660, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventBufferToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(661, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 381, [661 | __Ss], [__T | __Stack]);
yeccpars2(661, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(662, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 311, [662 | __Ss], [__T | __Stack]);
yeccpars2(662, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  mediaToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(663, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 291, [663 | __Ss], [__T | __Stack]);
yeccpars2(663, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 290, [663 | __Ss], [__T | __Stack]);
yeccpars2(663, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modemToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(664, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 279, [664 | __Ss], [__T | __Stack]);
yeccpars2(664, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  muxToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(665, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 575, [665 | __Ss], [__T | __Stack]);
yeccpars2(665, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  observedEventsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(666, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 698, [666 | __Ss], [__T | __Stack]);
yeccpars2(666, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  packagesToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(667, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 193, [667 | __Ss], [__T | __Stack]);
yeccpars2(667, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  signalsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(668, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 688, [668 | __Ss], [__T | __Stack]);
yeccpars2(668, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  statsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(669, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditItem,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(670, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 684, [670 | __Ss], [__T | __Stack]);
yeccpars2(670, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(685, __Cat, [670 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(671, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {digitMapDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(672, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {errorDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(673, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(674, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(675, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mediaDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(676, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {modemDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(677, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {muxDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(678, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {observedEventsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(679, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {packagesDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(680, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signalsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(681, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {statisticsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(682, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 683, [682 | __Ss], [__T | __Stack]);
yeccpars2(682, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(683, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(ammsReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(684, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 668, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 666, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 663, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 665, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 667, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 661, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 664, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 660, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 662, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(685, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_terminationAudit([__1|__2]),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(terminationAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(686, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 684, [686 | __Ss], [__T | __Stack]);
yeccpars2(686, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(687, __Cat, [686 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(687, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReturnParameterList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(688, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [688 | __Ss], [__T | __Stack]);
yeccpars2(688, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(689, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 696, [689 | __Ss], [__T | __Stack]);
yeccpars2(689, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StatisticsParameter'{statName = __1, statValue = asn1_NOVALUE},
 yeccpars2(yeccgoto(statisticsParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(690, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 691, [690 | __Ss], [__T | __Stack]);
yeccpars2(690, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(692, __Cat, [690 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(691, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [691 | __Ss], [__T | __Stack]);
yeccpars2(691, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(692, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 693, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(693, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(statisticsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(694, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 691, [694 | __Ss], [__T | __Stack]);
yeccpars2(694, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(695, __Cat, [694 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(695, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(statisticsParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(696, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 225, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(697, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StatisticsParameter'{statName = __1, statValue = [__3]},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(statisticsParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(698, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(699, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 701, [699 | __Ss], [__T | __Stack]);
yeccpars2(699, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(702, __Cat, [699 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(700, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_packagesItem(__1),
 yeccpars2(yeccgoto(packagesItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(701, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(702, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 703, [702 | __Ss], [__T | __Stack]);
yeccpars2(702, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(703, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(packagesDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(704, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 701, [704 | __Ss], [__T | __Stack]);
yeccpars2(704, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(705, __Cat, [704 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(705, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(packagesItems, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(706, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'ActionReply'.contextId,__5,__3),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(actionReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(707, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [707 | __Ss], [__T | __Stack]);
yeccpars2(707, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(708, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 709, [708 | __Ss], [__T | __Stack]);
yeccpars2(708, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeResParms,#'ServiceChangeResParm'{}},
 yeccpars2(710, __Cat, [708 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(709, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 711, [709 | __Ss], [__T | __Stack]);
yeccpars2(709, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [709 | __Ss], [__T | __Stack]);
yeccpars2(709, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(710, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeReply,#'ServiceChangeReply'{terminationID = [__3], serviceChangeResult = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(serviceChangeReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(711, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 716, [711 | __Ss], [__T | __Stack]);
yeccpars2(711, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(712, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 715, [712 | __Ss], [__T | __Stack]);
yeccpars2(712, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(713, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 714, [713 | __Ss], [__T | __Stack]);
yeccpars2(713, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(714, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeResParms,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(715, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {errorDescriptor,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(716, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 530, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 529, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 527, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 526, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(717, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(718, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 724, [718 | __Ss], [__T | __Stack]);
yeccpars2(718, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'ServiceChangeResParm'{},
 yeccpars2(725, __Cat, [718 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(719, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {address,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(720, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mgc_id,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(721, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {profile,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(722, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {version,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(723, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {time_stamp,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(724, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 530, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 529, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 527, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 526, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [724 | __Ss], [__T | __Stack]);
yeccpars2(724, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(725, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 726, [725 | __Ss], [__T | __Stack]);
yeccpars2(725, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(726, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeResParm(__3,__4),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(727, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 724, [727 | __Ss], [__T | __Stack]);
yeccpars2(727, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'ServiceChangeResParm'{},
 yeccpars2(728, __Cat, [727 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(728, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeResParm(__2,__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(servChgReplyParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(729, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(730, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 731, [730 | __Ss], [__T | __Stack]);
yeccpars2(730, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(732, __Cat, [730 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(731, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [731 | __Ss], [__T | __Stack]);
yeccpars2(731, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(732, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {notifyReply,#'NotifyReply'{terminationID = [__3], errorDescriptor = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(notifyReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(733, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 734, [733 | __Ss], [__T | __Stack]);
yeccpars2(733, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(734, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(notifyReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(735, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 736, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [735 | __Ss], [__T | __Stack]);
yeccpars2(735, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(736, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 742, [736 | __Ss], [__T | __Stack]);
yeccpars2(736, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(737, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditValueReply,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(738, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 739, [738 | __Ss], [__T | __Stack]);
yeccpars2(738, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditResult,#'AuditResult'{terminationID = __1, terminationAuditResult = []}},
 yeccpars2(yeccgoto(auditOther, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(739, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 668, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 666, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 663, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 665, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 667, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 661, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 664, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 660, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 662, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 115, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(740, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 741, [740 | __Ss], [__T | __Stack]);
yeccpars2(740, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(741, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditResult,#'AuditResult'{terminationID = __1, terminationAuditResult = __3}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditOther, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(742, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 745, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [742 | __Ss], [__T | __Stack]);
yeccpars2(742, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(743, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditValueReply,__4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(744, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {contextAuditResult,__1},
 yeccpars2(yeccgoto(contextTerminationAudit, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(745, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 748, [745 | __Ss], [__T | __Stack]);
yeccpars2(745, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(746, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 747, [746 | __Ss], [__T | __Stack]);
yeccpars2(746, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(747, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {contextAuditResult,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(contextTerminationAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(748, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(749, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 751, [749 | __Ss], [__T | __Stack]);
yeccpars2(749, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(750, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint(__1,0,999),
 yeccpars2(yeccgoto(errorCode, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(751, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 752, [751 | __Ss], [__T | __Stack]);
yeccpars2(751, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(753, __Cat, [751 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(752, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  value_of(__1),
 yeccpars2(yeccgoto(errorText, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(753, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 754, [753 | __Ss], [__T | __Stack]);
yeccpars2(753, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(754, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ErrorDescriptor'{errorCode = __3, errorText = __5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(errorDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(755, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 756, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [755 | __Ss], [__T | __Stack]);
yeccpars2(755, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(756, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 742, [756 | __Ss], [__T | __Stack]);
yeccpars2(756, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(757, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditCapReply,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(758, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditCapReply,__4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(759, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  'NULL',
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(optImmAckRequired, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(760, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [760 | __Ss], [__T | __Stack]);
yeccpars2(760, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(761, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 762, [761 | __Ss], [__T | __Stack]);
yeccpars2(761, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(762, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 763, [762 | __Ss], [__T | __Stack]);
yeccpars2(762, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(763, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionPending'{transactionId = ensure_transactionID(__3)},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionPending, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(764, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 765, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(765, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 766, [765 | __Ss], [__T | __Stack]);
yeccpars2(765, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainAddress(__2,asn1_NOVALUE),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(domainAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(766, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(767, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [767 | __Ss], [__T | __Stack]);
yeccpars2(767, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(768, __Cat, [767 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(768, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainAddress(__2,__5),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(domainAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(769, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 770, [769 | __Ss], [__T | __Stack]);
yeccpars2(769, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(770, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 771, [770 | __Ss], [__T | __Stack]);
yeccpars2(770, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainName(__2,asn1_NOVALUE),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(domainName, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(771, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(772, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [772 | __Ss], [__T | __Stack]);
yeccpars2(772, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(773, __Cat, [772 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(773, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainName(__2,__5),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(domainName, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(774, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'MegacoMessage'{authHeader = __2, mess = __3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(megacoMessage, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(775, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [775 | __Ss], [__T | __Stack]);
yeccpars2(775, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(776, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 777, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(777, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [777 | __Ss], [__T | __Stack]);
yeccpars2(777, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(778, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 779, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(779, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 99, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 98, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 97, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 94, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 93, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(780, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(781, __Cat, [780 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(781, __Cat, __Ss,  [__8,__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_auth_header(__3,__5,__7),
 __Nss = lists:nthtail(7, __Ss),
 yeccpars2(yeccgoto(authenticationHeader, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(__Other, _, _, _, _, _, _) ->
 exit({parser, __Other, missing_state_in_action_table}).

yeccgoto(actionReply, 622) ->
 624;
yeccgoto(actionReply, 628) ->
 630;
yeccgoto(actionReplyBody, 634) ->
 643;
yeccgoto(actionReplyList, 624) ->
 629;
yeccgoto(actionReplyList, 630) ->
 631;
yeccgoto(actionRequest, 130) ->
 132;
yeccgoto(actionRequest, 133) ->
 136;
yeccgoto(actionRequest, 600) ->
 607;
yeccgoto(actionRequest, 603) ->
 604;
yeccgoto(actionRequestItem, 141) ->
 154;
yeccgoto(actionRequestItem, 489) ->
 492;
yeccgoto(actionRequestItems, 154) ->
 490;
yeccgoto(actionRequestItems, 492) ->
 493;
yeccgoto(actionRequestList, 132) ->
 134;
yeccgoto(actionRequestList, 136) ->
 137;
yeccgoto(actionRequestList, 604) ->
 605;
yeccgoto(actionRequestList, 607) ->
 608;
yeccgoto(alternativeValue, 220) ->
 232;
yeccgoto(ammParameter, 169) ->
 179;
yeccgoto(ammParameter, 188) ->
 191;
yeccgoto(ammParameters, 179) ->
 189;
yeccgoto(ammParameters, 191) ->
 192;
yeccgoto(ammRequest, 141) ->
 155;
yeccgoto(ammRequest, 489) ->
 155;
yeccgoto(ammRequestBody, 168) ->
 170;
yeccgoto(ammToken, 141) ->
 156;
yeccgoto(ammToken, 489) ->
 156;
yeccgoto(ammsReply, 634) ->
 644;
yeccgoto(ammsReply, 652) ->
 644;
yeccgoto(ammsReplyBody, 657) ->
 659;
yeccgoto(ammsToken, 634) ->
 645;
yeccgoto(ammsToken, 652) ->
 645;
yeccgoto(auditDescriptor, 169) ->
 180;
yeccgoto(auditDescriptor, 188) ->
 180;
yeccgoto(auditDescriptor, 513) ->
 515;
yeccgoto(auditDescriptorBody, 471) ->
 482;
yeccgoto(auditItem, 471) ->
 483;
yeccgoto(auditItem, 484) ->
 486;
yeccgoto(auditItem, 658) ->
 669;
yeccgoto(auditItem, 684) ->
 669;
yeccgoto(auditItem, 739) ->
 669;
yeccgoto(auditItemList, 483) ->
 485;
yeccgoto(auditItemList, 486) ->
 487;
yeccgoto(auditOther, 735) ->
 737;
yeccgoto(auditOther, 755) ->
 757;
yeccgoto(auditReply, 634) ->
 646;
yeccgoto(auditReply, 652) ->
 646;
yeccgoto(auditRequest, 141) ->
 157;
yeccgoto(auditRequest, 489) ->
 157;
yeccgoto(auditReturnParameter, 658) ->
 670;
yeccgoto(auditReturnParameter, 684) ->
 686;
yeccgoto(auditReturnParameter, 739) ->
 670;
yeccgoto(auditReturnParameterList, 670) ->
 685;
yeccgoto(auditReturnParameterList, 686) ->
 687;
yeccgoto(authenticationHeader, 3) ->
 5;
yeccgoto(commandReply, 634) ->
 647;
yeccgoto(commandReply, 652) ->
 654;
yeccgoto(commandReplyList, 647) ->
 653;
yeccgoto(commandReplyList, 654) ->
 655;
yeccgoto(commandRequest, 141) ->
 158;
yeccgoto(commandRequest, 489) ->
 158;
yeccgoto(contextAudit, 141) ->
 159;
yeccgoto(contextAudit, 489) ->
 159;
yeccgoto(contextAuditProperties, 588) ->
 590;
yeccgoto(contextAuditProperties, 592) ->
 593;
yeccgoto(contextAuditProperty, 584) ->
 588;
yeccgoto(contextAuditProperty, 589) ->
 592;
yeccgoto(contextID, 138) ->
 139;
yeccgoto(contextID, 632) ->
 633;
yeccgoto(contextProperty, 141) ->
 160;
yeccgoto(contextProperty, 489) ->
 160;
yeccgoto(contextProperty, 634) ->
 648;
yeccgoto(contextProperty, 652) ->
 648;
yeccgoto(contextTerminationAudit, 736) ->
 743;
yeccgoto(contextTerminationAudit, 756) ->
 758;
yeccgoto(deviceName, 107) ->
 109;
yeccgoto(digitMapDescriptor, 169) ->
 181;
yeccgoto(digitMapDescriptor, 188) ->
 181;
yeccgoto(digitMapDescriptor, 658) ->
 671;
yeccgoto(digitMapDescriptor, 684) ->
 671;
yeccgoto(digitMapDescriptor, 739) ->
 671;
yeccgoto(domainAddress, 101) ->
 104;
yeccgoto(domainAddress, 551) ->
 104;
yeccgoto(domainAddress, 559) ->
 104;
yeccgoto(domainName, 101) ->
 105;
yeccgoto(domainName, 551) ->
 105;
yeccgoto(domainName, 559) ->
 105;
yeccgoto(embedFirst, 410) ->
 412;
yeccgoto(embedFirst, 414) ->
 416;
yeccgoto(embedNoSig, 392) ->
 397;
yeccgoto(embedNoSig, 405) ->
 397;
yeccgoto(embedSig, 429) ->
 433;
yeccgoto(embedSig, 437) ->
 433;
yeccgoto(embedWithSig, 392) ->
 398;
yeccgoto(embedWithSig, 405) ->
 398;
yeccgoto(errorCode, 748) ->
 749;
yeccgoto(errorDescriptor, 106) ->
 120;
yeccgoto(errorDescriptor, 569) ->
 571;
yeccgoto(errorDescriptor, 622) ->
 625;
yeccgoto(errorDescriptor, 634) ->
 649;
yeccgoto(errorDescriptor, 658) ->
 672;
yeccgoto(errorDescriptor, 684) ->
 672;
yeccgoto(errorDescriptor, 709) ->
 712;
yeccgoto(errorDescriptor, 731) ->
 733;
yeccgoto(errorDescriptor, 739) ->
 672;
yeccgoto(errorDescriptor, 742) ->
 746;
yeccgoto(errorText, 751) ->
 753;
yeccgoto(eventBufferControl, 327) ->
 330;
yeccgoto(eventBufferControl, 334) ->
 330;
yeccgoto(eventBufferControlState, 344) ->
 347;
yeccgoto(eventBufferDescriptor, 169) ->
 182;
yeccgoto(eventBufferDescriptor, 188) ->
 182;
yeccgoto(eventBufferDescriptor, 658) ->
 673;
yeccgoto(eventBufferDescriptor, 684) ->
 673;
yeccgoto(eventBufferDescriptor, 739) ->
 673;
yeccgoto(eventDM, 392) ->
 399;
yeccgoto(eventDM, 405) ->
 399;
yeccgoto(eventDM, 429) ->
 434;
yeccgoto(eventDM, 437) ->
 434;
yeccgoto(eventParameter, 392) ->
 400;
yeccgoto(eventParameter, 405) ->
 408;
yeccgoto(eventParameterName, 392) ->
 401;
yeccgoto(eventParameterName, 405) ->
 401;
yeccgoto(eventParameterName, 429) ->
 401;
yeccgoto(eventParameterName, 437) ->
 401;
yeccgoto(eventParameterName, 454) ->
 401;
yeccgoto(eventParameterName, 458) ->
 401;
yeccgoto(eventParameters, 400) ->
 406;
yeccgoto(eventParameters, 408) ->
 409;
yeccgoto(eventSpec, 445) ->
 446;
yeccgoto(eventSpec, 466) ->
 469;
yeccgoto(eventSpecList, 446) ->
 467;
yeccgoto(eventSpecList, 469) ->
 470;
yeccgoto(eventStreamOrOther, 392) ->
 402;
yeccgoto(eventStreamOrOther, 405) ->
 402;
yeccgoto(eventStreamOrOther, 429) ->
 435;
yeccgoto(eventStreamOrOther, 437) ->
 435;
yeccgoto(eventStreamOrOther, 454) ->
 456;
yeccgoto(eventStreamOrOther, 458) ->
 456;
yeccgoto(eventsDescriptor, 169) ->
 183;
yeccgoto(eventsDescriptor, 188) ->
 183;
yeccgoto(eventsDescriptor, 658) ->
 674;
yeccgoto(eventsDescriptor, 684) ->
 674;
yeccgoto(eventsDescriptor, 739) ->
 674;
yeccgoto(extension, 523) ->
 531;
yeccgoto(extension, 543) ->
 531;
yeccgoto(extensionParameter, 523) ->
 532;
yeccgoto(extensionParameter, 543) ->
 532;
yeccgoto(localControlDescriptor, 311) ->
 317;
yeccgoto(localControlDescriptor, 322) ->
 317;
yeccgoto(localControlDescriptor, 350) ->
 317;
yeccgoto(localControlDescriptor, 352) ->
 317;
yeccgoto(localParm, 357) ->
 361;
yeccgoto(localParm, 363) ->
 366;
yeccgoto(localParmList, 361) ->
 364;
yeccgoto(localParmList, 366) ->
 367;
yeccgoto(mId, 101) ->
 106;
yeccgoto(mId, 551) ->
 552;
yeccgoto(mId, 559) ->
 560;
yeccgoto(mediaDescriptor, 169) ->
 184;
yeccgoto(mediaDescriptor, 188) ->
 184;
yeccgoto(mediaDescriptor, 658) ->
 675;
yeccgoto(mediaDescriptor, 684) ->
 675;
yeccgoto(mediaDescriptor, 739) ->
 675;
yeccgoto(mediaParm, 311) ->
 318;
yeccgoto(mediaParm, 322) ->
 325;
yeccgoto(mediaParmList, 318) ->
 323;
yeccgoto(mediaParmList, 325) ->
 326;
yeccgoto(megacoMessage, 0) ->
 2;
yeccgoto(message, 5) ->
 100;
yeccgoto(messageBody, 106) ->
 121;
yeccgoto(modemDescriptor, 169) ->
 185;
yeccgoto(modemDescriptor, 188) ->
 185;
yeccgoto(modemDescriptor, 658) ->
 676;
yeccgoto(modemDescriptor, 684) ->
 676;
yeccgoto(modemDescriptor, 739) ->
 676;
yeccgoto(modemType, 290) ->
 309;
yeccgoto(modemType, 291) ->
 292;
yeccgoto(modemType, 294) ->
 307;
yeccgoto(modemTypeList, 292) ->
 295;
yeccgoto(modemTypeList, 307) ->
 308;
yeccgoto(mtpAddress, 107) ->
 110;
yeccgoto(muxDescriptor, 169) ->
 186;
yeccgoto(muxDescriptor, 188) ->
 186;
yeccgoto(muxDescriptor, 658) ->
 677;
yeccgoto(muxDescriptor, 684) ->
 677;
yeccgoto(muxDescriptor, 739) ->
 677;
yeccgoto(muxType, 279) ->
 280;
yeccgoto(notificationReason, 255) ->
 260;
yeccgoto(notificationReason, 261) ->
 264;
yeccgoto(notificationReasons, 260) ->
 262;
yeccgoto(notificationReasons, 264) ->
 265;
yeccgoto(notifyReply, 634) ->
 650;
yeccgoto(notifyReply, 652) ->
 650;
yeccgoto(notifyReplyBody, 730) ->
 732;
yeccgoto(notifyRequest, 141) ->
 161;
yeccgoto(notifyRequest, 489) ->
 161;
yeccgoto(notifyRequestBody, 569) ->
 572;
yeccgoto(observedEvent, 445) ->
 447;
yeccgoto(observedEvent, 466) ->
 447;
yeccgoto(observedEvent, 577) ->
 578;
yeccgoto(observedEvent, 579) ->
 582;
yeccgoto(observedEventBody, 449) ->
 455;
yeccgoto(observedEventBody, 464) ->
 465;
yeccgoto(observedEventParameter, 454) ->
 457;
yeccgoto(observedEventParameter, 458) ->
 461;
yeccgoto(observedEventParameters, 457) ->
 459;
yeccgoto(observedEventParameters, 461) ->
 462;
yeccgoto(observedEventTimeStamp, 445) ->
 448;
yeccgoto(observedEventTimeStamp, 466) ->
 448;
yeccgoto(observedEventTimeStamp, 577) ->
 448;
yeccgoto(observedEventTimeStamp, 579) ->
 448;
yeccgoto(observedEvents, 578) ->
 580;
yeccgoto(observedEvents, 582) ->
 583;
yeccgoto(observedEventsDescriptor, 569) ->
 573;
yeccgoto(observedEventsDescriptor, 658) ->
 678;
yeccgoto(observedEventsDescriptor, 684) ->
 678;
yeccgoto(observedEventsDescriptor, 739) ->
 678;
yeccgoto(onOrOff, 368) ->
 371;
yeccgoto(onOrOff, 372) ->
 373;
yeccgoto(optAuditDescriptor, 512) ->
 514;
yeccgoto(optAuditDescriptor, 595) ->
 596;
yeccgoto(optAuditDescriptor, 598) ->
 599;
yeccgoto(optImmAckRequired, 620) ->
 622;
yeccgoto(optParmValues, 296) ->
 298;
yeccgoto(optParmValues, 309) ->
 310;
yeccgoto(optSep, 0) ->
 3;
yeccgoto(optSep, 101) ->
 107;
yeccgoto(optSep, 109) ->
 114;
yeccgoto(optSep, 110) ->
 113;
yeccgoto(optSep, 448) ->
 463;
yeccgoto(optSep, 451) ->
 452;
yeccgoto(optSep, 551) ->
 107;
yeccgoto(optSep, 559) ->
 107;
yeccgoto(optSep, 767) ->
 768;
yeccgoto(optSep, 772) ->
 773;
yeccgoto(optSep, 780) ->
 781;
yeccgoto(packagesDescriptor, 658) ->
 679;
yeccgoto(packagesDescriptor, 684) ->
 679;
yeccgoto(packagesDescriptor, 739) ->
 679;
yeccgoto(packagesItem, 698) ->
 699;
yeccgoto(packagesItem, 701) ->
 704;
yeccgoto(packagesItems, 699) ->
 702;
yeccgoto(packagesItems, 704) ->
 705;
yeccgoto(parmValue, 213) ->
 224;
yeccgoto(parmValue, 299) ->
 306;
yeccgoto(parmValue, 401) ->
 404;
yeccgoto(parmValue, 532) ->
 548;
yeccgoto(parmValues, 300) ->
 302;
yeccgoto(parmValues, 304) ->
 305;
yeccgoto(pathName, 107) ->
 111;
yeccgoto(pkgdName, 193) ->
 196;
yeccgoto(pkgdName, 202) ->
 196;
yeccgoto(pkgdName, 271) ->
 196;
yeccgoto(pkgdName, 274) ->
 196;
yeccgoto(pkgdName, 297) ->
 299;
yeccgoto(pkgdName, 301) ->
 299;
yeccgoto(pkgdName, 327) ->
 299;
yeccgoto(pkgdName, 334) ->
 299;
yeccgoto(pkgdName, 357) ->
 299;
yeccgoto(pkgdName, 363) ->
 299;
yeccgoto(pkgdName, 384) ->
 385;
yeccgoto(pkgdName, 387) ->
 385;
yeccgoto(pkgdName, 421) ->
 422;
yeccgoto(pkgdName, 424) ->
 422;
yeccgoto(pkgdName, 445) ->
 449;
yeccgoto(pkgdName, 463) ->
 464;
yeccgoto(pkgdName, 466) ->
 449;
yeccgoto(pkgdName, 577) ->
 449;
yeccgoto(pkgdName, 579) ->
 449;
yeccgoto(pkgdName, 688) ->
 689;
yeccgoto(pkgdName, 691) ->
 689;
yeccgoto(portNumber, 551) ->
 553;
yeccgoto(portNumber, 766) ->
 767;
yeccgoto(portNumber, 771) ->
 772;
yeccgoto(priority, 141) ->
 162;
yeccgoto(priority, 489) ->
 162;
yeccgoto(priority, 634) ->
 162;
yeccgoto(priority, 652) ->
 162;
yeccgoto(propertyParm, 297) ->
 300;
yeccgoto(propertyParm, 301) ->
 304;
yeccgoto(propertyParm, 327) ->
 331;
yeccgoto(propertyParm, 334) ->
 331;
yeccgoto(propertyParm, 357) ->
 362;
yeccgoto(propertyParm, 363) ->
 362;
yeccgoto(requestID, 381) ->
 382;
yeccgoto(requestID, 419) ->
 420;
yeccgoto(requestID, 575) ->
 576;
yeccgoto(requestedEvent, 384) ->
 386;
yeccgoto(requestedEvent, 387) ->
 390;
yeccgoto(requestedEventBody, 385) ->
 393;
yeccgoto(requestedEvents, 386) ->
 388;
yeccgoto(requestedEvents, 390) ->
 391;
yeccgoto(safeToken, 5) ->
 101;
yeccgoto(safeToken, 102) ->
 769;
yeccgoto(safeToken, 103) ->
 764;
yeccgoto(safeToken, 107) ->
 112;
yeccgoto(safeToken, 129) ->
 601;
yeccgoto(safeToken, 138) ->
 140;
yeccgoto(safeToken, 166) ->
 167;
yeccgoto(safeToken, 193) ->
 197;
yeccgoto(safeToken, 202) ->
 197;
yeccgoto(safeToken, 207) ->
 213;
yeccgoto(safeToken, 215) ->
 213;
yeccgoto(safeToken, 220) ->
 226;
yeccgoto(safeToken, 221) ->
 226;
yeccgoto(safeToken, 222) ->
 226;
yeccgoto(safeToken, 223) ->
 226;
yeccgoto(safeToken, 230) ->
 226;
yeccgoto(safeToken, 231) ->
 226;
yeccgoto(safeToken, 235) ->
 226;
yeccgoto(safeToken, 236) ->
 226;
yeccgoto(safeToken, 246) ->
 247;
yeccgoto(safeToken, 266) ->
 267;
yeccgoto(safeToken, 268) ->
 269;
yeccgoto(safeToken, 271) ->
 197;
yeccgoto(safeToken, 274) ->
 197;
yeccgoto(safeToken, 279) ->
 281;
yeccgoto(safeToken, 282) ->
 167;
yeccgoto(safeToken, 285) ->
 167;
yeccgoto(safeToken, 290) ->
 293;
yeccgoto(safeToken, 291) ->
 293;
yeccgoto(safeToken, 294) ->
 293;
yeccgoto(safeToken, 297) ->
 197;
yeccgoto(safeToken, 301) ->
 197;
yeccgoto(safeToken, 327) ->
 197;
yeccgoto(safeToken, 334) ->
 197;
yeccgoto(safeToken, 348) ->
 247;
yeccgoto(safeToken, 357) ->
 197;
yeccgoto(safeToken, 363) ->
 197;
yeccgoto(safeToken, 381) ->
 383;
yeccgoto(safeToken, 384) ->
 197;
yeccgoto(safeToken, 387) ->
 197;
yeccgoto(safeToken, 392) ->
 403;
yeccgoto(safeToken, 405) ->
 403;
yeccgoto(safeToken, 419) ->
 383;
yeccgoto(safeToken, 421) ->
 197;
yeccgoto(safeToken, 424) ->
 197;
yeccgoto(safeToken, 429) ->
 403;
yeccgoto(safeToken, 437) ->
 403;
yeccgoto(safeToken, 445) ->
 450;
yeccgoto(safeToken, 454) ->
 403;
yeccgoto(safeToken, 458) ->
 403;
yeccgoto(safeToken, 463) ->
 197;
yeccgoto(safeToken, 466) ->
 450;
yeccgoto(safeToken, 494) ->
 167;
yeccgoto(safeToken, 498) ->
 167;
yeccgoto(safeToken, 503) ->
 167;
yeccgoto(safeToken, 511) ->
 167;
yeccgoto(safeToken, 517) ->
 167;
yeccgoto(safeToken, 523) ->
 533;
yeccgoto(safeToken, 543) ->
 533;
yeccgoto(safeToken, 549) ->
 550;
yeccgoto(safeToken, 551) ->
 554;
yeccgoto(safeToken, 555) ->
 226;
yeccgoto(safeToken, 557) ->
 558;
yeccgoto(safeToken, 561) ->
 562;
yeccgoto(safeToken, 563) ->
 564;
yeccgoto(safeToken, 565) ->
 566;
yeccgoto(safeToken, 567) ->
 167;
yeccgoto(safeToken, 575) ->
 383;
yeccgoto(safeToken, 577) ->
 450;
yeccgoto(safeToken, 579) ->
 450;
yeccgoto(safeToken, 594) ->
 167;
yeccgoto(safeToken, 597) ->
 167;
yeccgoto(safeToken, 610) ->
 611;
yeccgoto(safeToken, 613) ->
 611;
yeccgoto(safeToken, 618) ->
 601;
yeccgoto(safeToken, 632) ->
 140;
yeccgoto(safeToken, 656) ->
 167;
yeccgoto(safeToken, 688) ->
 197;
yeccgoto(safeToken, 691) ->
 197;
yeccgoto(safeToken, 696) ->
 226;
yeccgoto(safeToken, 698) ->
 700;
yeccgoto(safeToken, 701) ->
 700;
yeccgoto(safeToken, 707) ->
 167;
yeccgoto(safeToken, 716) ->
 717;
yeccgoto(safeToken, 724) ->
 717;
yeccgoto(safeToken, 729) ->
 167;
yeccgoto(safeToken, 735) ->
 167;
yeccgoto(safeToken, 742) ->
 167;
yeccgoto(safeToken, 748) ->
 750;
yeccgoto(safeToken, 755) ->
 167;
yeccgoto(safeToken, 760) ->
 601;
yeccgoto(safeToken, 766) ->
 554;
yeccgoto(safeToken, 771) ->
 554;
yeccgoto(safeToken, 775) ->
 776;
yeccgoto(safeToken, 777) ->
 778;
yeccgoto(safeToken, 779) ->
 780;
yeccgoto(secondEventParameter, 429) ->
 436;
yeccgoto(secondEventParameter, 437) ->
 440;
yeccgoto(secondEventParameters, 436) ->
 438;
yeccgoto(secondEventParameters, 440) ->
 441;
yeccgoto(secondRequestedEvent, 421) ->
 423;
yeccgoto(secondRequestedEvent, 424) ->
 427;
yeccgoto(secondRequestedEventBody, 422) ->
 430;
yeccgoto(secondRequestedEvents, 423) ->
 425;
yeccgoto(secondRequestedEvents, 427) ->
 428;
yeccgoto(servChgReplyParm, 716) ->
 718;
yeccgoto(servChgReplyParm, 724) ->
 727;
yeccgoto(servChgReplyParms, 718) ->
 725;
yeccgoto(servChgReplyParms, 727) ->
 728;
yeccgoto(serviceChangeAddress, 523) ->
 534;
yeccgoto(serviceChangeAddress, 543) ->
 534;
yeccgoto(serviceChangeAddress, 716) ->
 719;
yeccgoto(serviceChangeAddress, 724) ->
 719;
yeccgoto(serviceChangeDelay, 523) ->
 535;
yeccgoto(serviceChangeDelay, 543) ->
 535;
yeccgoto(serviceChangeDescriptor, 519) ->
 521;
yeccgoto(serviceChangeMethod, 523) ->
 536;
yeccgoto(serviceChangeMethod, 543) ->
 536;
yeccgoto(serviceChangeMgcId, 523) ->
 537;
yeccgoto(serviceChangeMgcId, 543) ->
 537;
yeccgoto(serviceChangeMgcId, 716) ->
 720;
yeccgoto(serviceChangeMgcId, 724) ->
 720;
yeccgoto(serviceChangeParm, 523) ->
 538;
yeccgoto(serviceChangeParm, 543) ->
 546;
yeccgoto(serviceChangeParms, 538) ->
 544;
yeccgoto(serviceChangeParms, 546) ->
 547;
yeccgoto(serviceChangeProfile, 523) ->
 539;
yeccgoto(serviceChangeProfile, 543) ->
 539;
yeccgoto(serviceChangeProfile, 716) ->
 721;
yeccgoto(serviceChangeProfile, 724) ->
 721;
yeccgoto(serviceChangeReason, 523) ->
 540;
yeccgoto(serviceChangeReason, 543) ->
 540;
yeccgoto(serviceChangeReply, 634) ->
 651;
yeccgoto(serviceChangeReply, 652) ->
 651;
yeccgoto(serviceChangeReplyBody, 708) ->
 710;
yeccgoto(serviceChangeReplyDescriptor, 709) ->
 713;
yeccgoto(serviceChangeRequest, 141) ->
 163;
yeccgoto(serviceChangeRequest, 489) ->
 163;
yeccgoto(serviceChangeVersion, 523) ->
 541;
yeccgoto(serviceChangeVersion, 543) ->
 541;
yeccgoto(serviceChangeVersion, 716) ->
 722;
yeccgoto(serviceChangeVersion, 724) ->
 722;
yeccgoto(serviceState, 339) ->
 343;
yeccgoto(serviceStates, 327) ->
 332;
yeccgoto(serviceStates, 334) ->
 332;
yeccgoto(sigParameter, 207) ->
 214;
yeccgoto(sigParameter, 215) ->
 218;
yeccgoto(sigParameters, 214) ->
 216;
yeccgoto(sigParameters, 218) ->
 219;
yeccgoto(signalList, 193) ->
 198;
yeccgoto(signalList, 202) ->
 198;
yeccgoto(signalListId, 268) ->
 270;
yeccgoto(signalListParm, 271) ->
 272;
yeccgoto(signalListParm, 274) ->
 277;
yeccgoto(signalListParms, 272) ->
 275;
yeccgoto(signalListParms, 277) ->
 278;
yeccgoto(signalName, 193) ->
 199;
yeccgoto(signalName, 202) ->
 199;
yeccgoto(signalName, 271) ->
 199;
yeccgoto(signalName, 274) ->
 199;
yeccgoto(signalParm, 193) ->
 200;
yeccgoto(signalParm, 202) ->
 205;
yeccgoto(signalParms, 200) ->
 203;
yeccgoto(signalParms, 205) ->
 206;
yeccgoto(signalRequest, 193) ->
 201;
yeccgoto(signalRequest, 202) ->
 201;
yeccgoto(signalRequest, 271) ->
 273;
yeccgoto(signalRequest, 274) ->
 273;
yeccgoto(signalType, 249) ->
 253;
yeccgoto(signalsDescriptor, 169) ->
 187;
yeccgoto(signalsDescriptor, 188) ->
 187;
yeccgoto(signalsDescriptor, 410) ->
 413;
yeccgoto(signalsDescriptor, 442) ->
 443;
yeccgoto(signalsDescriptor, 658) ->
 680;
yeccgoto(signalsDescriptor, 684) ->
 680;
yeccgoto(signalsDescriptor, 739) ->
 680;
yeccgoto(statisticsDescriptor, 658) ->
 681;
yeccgoto(statisticsDescriptor, 684) ->
 681;
yeccgoto(statisticsDescriptor, 739) ->
 681;
yeccgoto(statisticsParameter, 688) ->
 690;
yeccgoto(statisticsParameter, 691) ->
 694;
yeccgoto(statisticsParameters, 690) ->
 692;
yeccgoto(statisticsParameters, 694) ->
 695;
yeccgoto(streamDescriptor, 311) ->
 319;
yeccgoto(streamDescriptor, 322) ->
 319;
yeccgoto(streamID, 246) ->
 248;
yeccgoto(streamID, 348) ->
 349;
yeccgoto(streamModes, 374) ->
 380;
yeccgoto(streamParm, 311) ->
 320;
yeccgoto(streamParm, 322) ->
 320;
yeccgoto(streamParm, 350) ->
 351;
yeccgoto(streamParm, 352) ->
 355;
yeccgoto(streamParmList, 351) ->
 353;
yeccgoto(streamParmList, 355) ->
 356;
yeccgoto(subtractRequest, 141) ->
 164;
yeccgoto(subtractRequest, 489) ->
 164;
yeccgoto(terminationA, 494) ->
 495;
yeccgoto(terminationA, 498) ->
 495;
yeccgoto(terminationAudit, 658) ->
 682;
yeccgoto(terminationAudit, 739) ->
 740;
yeccgoto(terminationB, 503) ->
 504;
yeccgoto(terminationID, 166) ->
 168;
yeccgoto(terminationID, 282) ->
 284;
yeccgoto(terminationID, 285) ->
 288;
yeccgoto(terminationID, 494) ->
 496;
yeccgoto(terminationID, 498) ->
 496;
yeccgoto(terminationID, 503) ->
 505;
yeccgoto(terminationID, 511) ->
 512;
yeccgoto(terminationID, 517) ->
 518;
yeccgoto(terminationID, 567) ->
 568;
yeccgoto(terminationID, 594) ->
 595;
yeccgoto(terminationID, 597) ->
 598;
yeccgoto(terminationID, 656) ->
 657;
yeccgoto(terminationID, 707) ->
 708;
yeccgoto(terminationID, 729) ->
 730;
yeccgoto(terminationID, 735) ->
 738;
yeccgoto(terminationID, 742) ->
 284;
yeccgoto(terminationID, 755) ->
 738;
yeccgoto(terminationIDList, 280) ->
 283;
yeccgoto(terminationIDList, 736) ->
 744;
yeccgoto(terminationIDList, 756) ->
 744;
yeccgoto(terminationIDListRepeat, 284) ->
 286;
yeccgoto(terminationIDListRepeat, 288) ->
 289;
yeccgoto(terminationStateDescriptor, 311) ->
 321;
yeccgoto(terminationStateDescriptor, 322) ->
 321;
yeccgoto(terminationStateParm, 327) ->
 333;
yeccgoto(terminationStateParm, 334) ->
 337;
yeccgoto(terminationStateParms, 333) ->
 335;
yeccgoto(terminationStateParms, 337) ->
 338;
yeccgoto(timeStamp, 445) ->
 451;
yeccgoto(timeStamp, 466) ->
 451;
yeccgoto(timeStamp, 523) ->
 542;
yeccgoto(timeStamp, 543) ->
 542;
yeccgoto(timeStamp, 577) ->
 451;
yeccgoto(timeStamp, 579) ->
 451;
yeccgoto(timeStamp, 716) ->
 723;
yeccgoto(timeStamp, 724) ->
 723;
yeccgoto(topologyDescriptor, 141) ->
 165;
yeccgoto(topologyDescriptor, 489) ->
 165;
yeccgoto(topologyDescriptor, 634) ->
 165;
yeccgoto(topologyDescriptor, 652) ->
 165;
yeccgoto(topologyDirection, 506) ->
 510;
yeccgoto(topologyTriple, 494) ->
 497;
yeccgoto(topologyTriple, 498) ->
 501;
yeccgoto(topologyTripleList, 497) ->
 499;
yeccgoto(topologyTripleList, 501) ->
 502;
yeccgoto(transactionAck, 610) ->
 612;
yeccgoto(transactionAck, 613) ->
 616;
yeccgoto(transactionAckList, 612) ->
 614;
yeccgoto(transactionAckList, 616) ->
 617;
yeccgoto(transactionID, 129) ->
 602;
yeccgoto(transactionID, 618) ->
 619;
yeccgoto(transactionID, 760) ->
 761;
yeccgoto(transactionItem, 106) ->
 122;
yeccgoto(transactionItem, 122) ->
 122;
yeccgoto(transactionList, 106) ->
 123;
yeccgoto(transactionList, 122) ->
 128;
yeccgoto(transactionPending, 106) ->
 124;
yeccgoto(transactionPending, 122) ->
 124;
yeccgoto(transactionReply, 106) ->
 125;
yeccgoto(transactionReply, 122) ->
 125;
yeccgoto(transactionReplyBody, 622) ->
 626;
yeccgoto(transactionRequest, 106) ->
 126;
yeccgoto(transactionRequest, 122) ->
 126;
yeccgoto(transactionResponseAck, 106) ->
 127;
yeccgoto(transactionResponseAck, 122) ->
 127;
yeccgoto(value, 220) ->
 233;
yeccgoto(value, 221) ->
 229;
yeccgoto(value, 222) ->
 228;
yeccgoto(value, 223) ->
 227;
yeccgoto(value, 230) ->
 243;
yeccgoto(value, 231) ->
 234;
yeccgoto(value, 235) ->
 241;
yeccgoto(value, 236) ->
 239;
yeccgoto(value, 555) ->
 556;
yeccgoto(value, 696) ->
 697;
yeccgoto(valueList, 234) ->
 237;
yeccgoto(valueList, 239) ->
 240;
yeccgoto(valueList, 243) ->
 244;
yeccgoto(__Symbol, __State) ->
 exit({__Symbol, __State, missing_in_goto_table}).


